/*------------------------------------------------------------------------------------------------------*\
|   PROGRAM NAME: rx_prev_summ.sas                                                                       |
|     Creates macro rx_prev_summ for prevalent RX summary tables: Drug Class and Generic Name            |
|--------------------------------------------------------------------------------------------------------|
|                                                                                                        |
|   Version: Revised 1.0                                                                                 |
|   Date: October 11, 2015                                                                               |
|   MOD 10/18/15: Modified use of macro variable "Age_vs_Period"                                         |
|   MOD 10/20/15: For quarterly aggregates, use AgeQuarterKey variable instead of AgeTimePeriodGroupKey  |
|--------------------------------------------------------------------------------------------------------|
|                                                                                                        |
|   Program inputs:                                                                                      |
|       Multiple tables named RxXXXYYY1, RxXXXYYY2, RxXXXYYY3, etc. where 'XXX' is either                |
|          DrugClass or GenericName                                                                      |
|       NDC lookup table with 1:1 drug class keys and drug class descriptions                            |
|   Program outputs:                                                                                     |
|       Prevalent Drug Class  or Generic Name Summary Table                                              |
|--------------------------------------------------------------------------------------------------------|
|                                                                                                        |
|   Contact:                                                                                             |
|       Mini-Sentinel Coordinating Center                                                                |
|       info@mini-sentinel.org                                                                           |
\*------------------------------------------------------------------------------------------------------*/

%macro rx_prev_summ(
  RxType=,
  OutputName=,
  DescriptionLength=,
  DescriptionLabel=
  );
  %*** Declare local macro vars;
  %local year ;
  %do year=&DP_StartYear. %to &DP_EndYear.;
    proc sql;
      *** Aggregate by Age_Group, Sex, Period, and Rx Type, for unique dispensings;
      %** DEV Note: order group-by vars from most to least number of distinct values for best performance;
      create table Rx&RxType.&Year. as
      select AgeTimePeriodGroupKey, Sex, Period, &RxType.Key
        , count(distinct PatKey) as Members, count(*) as Dispensings, sum(RxSup) as DaysSupply
      from DPTemp.Rx&RxType.&Year.
      group by &RxType.Key, %insertchar(&Age_vs_Period.), Sex
      ;
      *** Aggregate again but by Age_Group, Sex, Quarter, and Rx Type, for unique dispensings;
      %** DEV Note: order group-by vars from most to least number of distinct values for best performance;
      create table Rx&RxType.Quarter&Year. as
      select AgeQuarterKey as AgeTimePeriodGroupKey, Sex, Quarter as Period, &RxType.Key
        , count(distinct PatKey) as Members, count(*) as Dispensings, sum(RxSup) as DaysSupply
      from DPTemp.Rx&RxType.&Year.
      group by &RxType.Key,
        %if %eval(&DP_NumYears. * 4) > 10 %then %do;
          Quarter, AgeQuarterKey,
        %end;
        %else %do;
          AgeQuarterKey, Quarter,
        %end;
      Sex
      ;
    quit;
  %end;

  %*** Stack all periods together and merge with lookup table to get descriptions;
  proc sql ;
    create table DPLocal.&OutputName. as
      select put(AgeTimePeriodGroupKey,$AgeGroupID_to_Value.) as Age_Group length=5 format=$5. label="Age Group"
      , Sex	length=1 format=$1. label="Sex"
      , Period length=6 format=$6. label="Year/Quarter"
      , &RxType. length=&DescriptionLength. format=$&DescriptionLength.. label="&DescriptionLabel."
      , Members length=8 format=15. label="# of Members"
      , Dispensings	length=8 format=15. label="# of Dispensings"
      , DaysSupply length=8 format=15. label="Days Supply"
      , input(put(AgeTimePeriodGroupKey,$AgeGroupID_to_True_ID.),2.) as Age_Group_ID length=3 format=2. label="ID"
          from (
      %do year=&DP_StartYear. %to &DP_EndYear.;
        select * from Rx&RxType.&Year.
        union corr all
        select * from Rx&RxType.Quarter&Year.
        %if &Year. NE &DP_EndYear. %then %do;
          union corr all
        %end;
      %end;) as d
    , DPTemp.&RxType.lookup as lk
    where d.&RxType.Key = lk.&RxType.Key
    order by Age_Group_ID, Sex, Period, &RxType.
    ;
  quit;

%mend;

* End of rx_prev_summ.sas;