/*---------------------------------------------------------------------------------------------------*\
|   Program name: stockpile_rx (forward rx filling) Tool.                                             |
|                                                                                                     |
|   Date: November 2015                                                                               |
|   Version: 1.0                                                                                      |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   This program is an adaption of the MS Stockpiling ToolKit Macro                                   |
|                                                                                                     |
|   This program converts overlapping drug dispensings into non-overlapping dispensings. In this      |
|   version, only the UNCONDITIONAL stockpiling algorithm is supported.                               |
|                                                                                                     |
|   UNCONDITIONAL STOCKPILING:                                                                        |
|   -------------------------                                                                         |
|   For the unconditional stockpiling algorithm, the dispensing date of each overlapping claim        |
|   is always pushed forward to the end of days of supply of the previous dispensing                  |
|                                                                                                     |
|   For the unconditional algorithm, the valid fill status values below ARE mutually exclusive:       |
|     'F' (First fill)      -> A first fill within a set of &PATID &GROUP records                     |
|     'G' (Gap fill)        -> A subsequent fill that does NOT overlap with the prior fill            |
|     '+' (Stockpile fill)  -> A subsequent fill that DOES overlap with the prior fill. The fill      |
|                              date is unconditionally pushed forward to one day after the end of days|
|                              of supply of the previous fill.                                        |
|                                                                                                     |
|                                                                                                     |
|   CLEANING INPUT DATA                                                                               |
|   -------------------                                                                               |
|   Regardless of which stockpiling algorithm used, it is expected that the input data is cleaned for |
|   invalid claims (e.g. missing values/out of range values) and that same-day claims are aggregated. |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Program inputs:                                                                                   |
|                  -SAS data file with drug claims data                                               |
|                    - File must have variables &PATID. &GROUPING. &CLMDATE. &CLMSUP.                 |
|                    - File must be appropriately cleaned (see above)                                 |
|                  -Utility macros DSN_EXISTS and DSN_OBS have been previously been included          |
|                                                                                                     |
|   Program outputs:                                                                                  |
|                  -SAS data file (.SAS7BDAT format) containing the stockpiled claims                 |
|                                                                                                     |
|   Macro variable definitions:                                                                       |
|     INFILE      = Input file with drug claims data                                                  |
|     PATID       = Name of variable in INFILE to identify the patient-ID (defaults to PatID)         |
|     GROUPING    = Variables in INFILE used to group claims                                          |
|     CLMDATE     = Name of variable in INFILE to identify the claim date                             |
|     CLMSUP      = Name of variable in INFILE to identify the claim days of supply                   |
|     CLMAMT      = Name of variable in INFILE to identify the claim amount dispensed (OPTIONAL)      |
|     WHERE       = Condition for applicable rows in INFILE (OPTIONAL)                                |
|     OUTFILE     = Output file                                                                       |
|     DEBUG       = Y/N flag for producing additional debug output (defaults to N)                    |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Contact:                                                                                          |
|       Mini-Sentinel Coordinating Center                                                             |
|       info@mini-sentinel.org                                                                        |
|                                                                                                     |
\*---------------------------------------------------------------------------------------------------*/

/* MOD: 10/14/15 -MR: */
/*                1) Modify to drop entirely truncated claims */
/*                2) Modify to recursively re-stockpile to remove residual overlaps remaining */
/*                   using conditional algorithm with 0 days allowed overlap */
/* MOD: 11/09/15 -MR: */
/*                1) Modify to remove CONDITIONAL algorithm from summary tables package as it is not */
/*                   needed and it will increase resources needed for QA review */

%macro STOCKPILE_RX ( INFILE=,
                      PATID=,
                      GROUPING=,
                      CLMDATE=,
                      CLMSUP=,
                      CLMAMT=,
                      WHERE=,
                      OUTFILE=,
                      DEBUG=N
                    );
  /*Set default value for PATID if left blank*/
  %if %length(&PATID)=0 %then %do ;
     %let PATID = PatID ;
  %end ;

  %put *** Display Macro Parameter Values;
  %put _local_ ;
  %put ;

  %put  *** Derive the group-by variables used for stockpiling;
  %local STOCKDIMS ;
  %let STOCKDIMS = &PATID &GROUPING.;
  %put &=STOCKDIMS.;
  %put ;

  %put  *** Derive right-most variable in stockpiling group-by variables;
  %local DIMCNT LASTDIM;
  %let DIMCNT = %SYSFUNC(COUNTW(&STOCKDIMS.));
  %let LASTDIM = %SCAN(&STOCKDIMS.,&DIMCNT.) ;
  %put &=LASTDIM.;
  %put ;

  /* Track if WHERE clause has been applied yet */
  %local WHERE_Applied ;
  %let WHERE_Applied = 0 ;

  /* WhichFile local macro variable is conditionally set to &OUTFILE or &INFILE */
  /*   WhichFile is set to &OUTFILE when */
  /*     - &INFILE is NOT sorted properly */
  /*   WhichFile is set to &INFILE when: */
  /*     - &INFILE IS sorted properly */
  %local WhichFile ;
  %put  *** Verify &INFILE sort order;
  %local SORTEDBY N_SORTED INFILE_VARS;
  %let N_SORTED = %SYSFUNC(COUNTW(&PATID. &GROUPING. &CLMDATE.));
  proc contents noprint data=&INFILE. out=_contents_ (keep=name sortedby varnum) ;
  run ;
  proc sql noprint ;
    select upcase(name) into :SORTEDBY separated by ' '
       from _contents_
       where sortedby between 1 and &N_SORTED.
       order by sortedby
    ;
    select name into :INFILE_VARS separated by ' '
       from _contents_
       order by varnum
    ;
  quit ;
  %put ;
  %put INFILE VARS: &=INFILE_VARS ;
  %put INFILE SORTEDBY VARS: &=SORTEDBY ;
  %put ;
  %if %upcase(%cmpres(&SORTEDBY)) EQ %upcase(%cmpres(&PATID. &GROUPING. &CLMDATE.))  %then %do;
      %put  *** &INFILE is already sorted in the proper order;
      %let WhichFile=&INFILE. ;
  %end;
  %else %do ;
     %put  *** Sort &INFILE by &PATID. &GROUPING. &CLMDATE. outputting to &OUTFILE.;
     %put &=WhichFile &=INFILE &=WHERE ;
     proc sort data=&INFILE. out=&OUTFILE. ;
       by &PATID. &GROUPING. &CLMDATE. ;
       %if %length(&WHERE.)>0 %then %do;
         where &WHERE. ;  %let WHERE_Applied=1 ;
       %end;
     run ;
    %let WhichFile=&OUTFILE. ;
  %end ;

  /* WhichFile local macro variable is reset after steps below to &OUTFILE when: */
  /*   - &DEBUG is Y */
  %if %upcase(&DEBUG.) eq Y %then %do;
     %put  *** Step to add DEBUG fields to claims ;
     data &OUTFILE.;
       set &WhichFile.;
       %if %length(&WHERE.)>0 and &WHERE_Applied.=0 %then %do;
         where &WHERE. ;  %let WHERE_Applied=1 ;
       %end;
       format Orig&CLMDATE OrigExpireDt mmddyy10. ;
       Orig&CLMDATE = &CLMDATE ;
       OrigExpireDt=&CLMDATE.+(&CLMSUP.-1);
       Orig&CLMSUP. = &CLMSUP. ;
       %if %length(&CLMAMT)>0 %then %do ;
          Orig&CLMAMT. = &CLMAMT. ;
       %end ;
     run ;
     %let WhichFile=&OUTFILE ;
  %end;

  %if %upcase(&DEBUG.) eq Y %then %do;
    PROC PRINT DATA=&OUTFILE (obs=10) ;
       TITLE "STEP 1 - &OUTFILE" ;
    RUN ;
  %end ;

  %put  *** Stockpile claims;
  data &OUTFILE. (sortedby=&PATID &GROUPING &CLMDATE.);
  set &WhichFile.;
  %if %length(&WHERE.)>0 and &WHERE_Applied.=0 %then %do;
     where &WHERE. ;  %let WHERE_Applied=1 ;
  %end;

  by &PATID &GROUPING.;

  format &CLMDATE. ExpireDt lexpiredt mmddyy10.;

  if first.&LASTDIM. then WithinGroupRowID=1;
  else WithinGroupRowID+1;

  ExpireDt=&CLMDATE.+(&CLMSUP.-1);

  if first.&LASTDIM. then do ;
     lexpiredt=.;
     l&CLMSUP.=.;
     overlap_prior=.;  /*Note: Overlap is cumulative within a series of stockpiled claims*/
  end ;
  else do ;
    overlap_prior=lexpiredt-&CLMDATE.+1;
  end ;

  attrib FillStatus length=$3 label='First Fill (F), Gap Fill (G), Truncated Fill (-), Early Fill (E), Stockpiling Fill (+)';
  if first.&LASTDIM. then FillStatus = 'F' ;

  if overlap_prior > 0  then do;
    PercentDays_prior = (overlap_prior/L&CLMSUP.);
    FillStatus = '+' ;
    &CLMDATE.=lexpiredt+1;
    expireDt=&CLMDATE.+(&CLMSUP.-1);
  end;
  else do ;
    if NOT first.&LASTDIM. then FillStatus = 'G' ;
  end ;

  lexpiredt=ExpireDt;
  l&CLMSUP.=&CLMSUP.;

  retain lexpiredt L&CLMSUP. ;
  %if %upcase(&DEBUG.) eq N %then %do;
    drop lexpiredt l&CLMSUP.;
    drop WithinGroupRowID overlap_prior PercentDays_prior ;
  %end;
  run;

  %if %upcase(&DEBUG.) eq Y %then %do;
    PROC PRINT DATA=&OUTFILE (obs=10) ;
       TITLE "STEP 2 - &OUTFILE" ;
       TITLE2 "Note: Overlap is cumulative within a series of stockpiled claims" ;
    RUN ;
  %end ;

  %if %upcase(&DEBUG.) eq N %then %do;
     proc datasets library = work nolist nowarn nodetails;
       delete _CONTENTS ;
     quit;
  %end;

%mend STOCKPILE_RX;

* End of stockpile_rx.sas;