/*---------------------------------------------------------------------------------------------------*\
|   PROGRAM NAME: user_functions_formats.sas                                                          |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Version: Revised 1.0                                                                              |
|   Date: August 27, 2015                                                                             |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Program inputs:                                                                                   |
|       NONE                                                                                          |
|                                                                                                     |
|   Program outputs:                                                                                  |
|       User defined functions and formats                                                            |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Contact:                                                                                          |
|       Mini-Sentinel Coordinating Center                                                             |
|       info@mini-sentinel.org                                                                        |
\*---------------------------------------------------------------------------------------------------*/


%**************** BEGIN LIST OF FUNCTIONS/FORMATS INCLUDED **************************************;
%*;
%*  FUNCTIONS:;
%*  ----------;
%*   AgeCalc             -- Calculates age;
%*   NumLength           -- Calculates minimal # of bytes needed to accurately store an interger;
%*
%*  FORMATS:;
%*  ----------;
%*   AgeGroup_to_ID      -- Maps numeric ages to character age-group-ids;
%*   $AgeGroup_to_ID     -- Maps character age-groups to character age-group-ids;
%*   $AgeGroup_to_Value  -- Maps character age-group-ids to character age-group descriptions;
%*;
%**************** END LIST OF FUNCTIONS/FORMATS NCLUDED *****************************************;


*** Set DsnFErr option to avoid triggering a SAS bug related to creating FCMP functions;
%let _DsnFErr_ = %sysfunc(getoption(DsnFErr));
options DsnFErr; run;

*** Blow away DPTEMP.FCMP_Functions SAS dataset for good measure;
proc datasets lib=DPTemp nolist nowarn nodetails;
  delete FCMP_Functions;
quit;

*** Assign dataset DPTEMP.FCMP_Functions as repository for user created FCMP functions;
options cmplib=DPTEMP.FCMP_Functions ;
run ;

*** Create user-defined functions
*** Requires that a SAS OPTIONS statement include this assignment: "cmplib=DPTEMP.FCMP_Functions" (no quotes);
proc fcmp outlib =DPTEMP.FCMP_Functions.one;

  ** Function to calculate age to exact year;
  function AgeCalc(DOB, AsOfDate);
    * DOB: Date of birth as SAS date only (no date/time values)
    * AsOfDate: Date as of when to calculate age (no date/time values);
    _Age=floor((intck("MONTH", DOB, AsOfDate) - (day(AsOfDate) < day(DOB))) / 12) ;
    return(_Age);
  endsub;

  ** Function to determine the minimum number of bytes to hold a numeric integer;
  function NumLength (NumIn);
    ** Check for non-integers and negative values;
    if NumIn < 0 or (int(NumIn) NE NumIn) then do;
      _val=8;
    end;
    else do;
      select;
        when (0 <= NumIn <= 8192) _val=3;
        when (8193 <= NumIn <= 2097152) _val=4;
        when (2097153 <= NumIn <= 536870912) _val=5;
        when (536870913 <= NumIn <= 137438953472) _val=6;
        when (137438953473 <= NumIn <= 35184372088832) _val=7;
        otherwise _val=8;
      end;
    end;
    return(_val);
  endsub;

run;

*** Restore DsnFErr option;
options &_DsnFErr_; run;
%symdel _DsnFErr_;


*** Create user-defined formats;
proc format;
  * Maps a numeric age value to an AgeGroup ID value;
  * MOD 10/21/15: Change range of '75-120' to '75-HIGH';
  value AgeGroup_to_ID
    0-1   = "1"
    2-4   = "2"
    5-9   = "3"
    10-14 = "4"
    15-18 = "5"
    19-21 = "6"
    22-44 = "7"
    45-64 = "8"
    65-74 = "9"
    75-HIGH = "0"
    Other  = "X"
  ;
  * Maps a character AgeGroup value to an AgeGroup ID value;
  value $AgeGroup_to_ID
    "0-1"   = "1"
    "2-4"   = "2"
    "5-9"   = "3"
    "10-14" = "4"
    "15-18" = "5"
    "19-21" = "6"
    "22-44" = "7"
    "45-64" = "8"
    "65-74" = "9"
    "75+"   = "0"
    Other   = "X"
  ;
  * Maps surrogate character AgeGroup ID value to an AgeGroup descriptive value;
  value $AgeGroupID_to_Value
    "1" = "0-1"
    "2" = "2-4"
    "3" = "5-9"
    "4" = "10-14"
    "5" = "15-18"
    "6" = "19-21"
    "7" = "22-44"
    "8" = "45-64"
    "9" = "65-74"
    "0" = "75+"
    Other   = "X"
  ;
  * Maps surrogate character AgeGroup ID value to true AgeGroup ID value;
  value $AgeGroupID_to_True_ID
    "1" = "1"
    "2" = "2"
    "3" = "3"
    "4" = "4"
    "5" = "5"
    "6" = "6"
    "7" = "7"
    "8" = "8"
    "9" = "9"
    "0" = "10"
    Other= "99"
  ;
run;

* End of user_functions_formats.sas;